#!/usr/bin/env perl
use v5.38;

use strict;
use warnings;

use Carp;
use Data::Dumper;
use Time::HiRes qw(time);

BEGIN {
    unshift @INC, './lib';
};

use ColorDemo::Sound;
use ColorDemo::Screen;
use ColorDemo::Font;
use ColorDemo::Intro;
use ColorDemo::Scroller;
use ColorDemo::Story;

my $start = time;
my $screen = ColorDemo::Screen->new();
$screen->clearBuffer(0, 0, 0);
$screen->setStatus("Loading...");
$screen->render();

my $sound = ColorDemo::Sound->new();
my $font = ColorDemo::Font->new(screen => $screen);
my $intro = ColorDemo::Intro->new(screen => $screen, font => $font, sound => $sound);
my $scroller = ColorDemo::Scroller->new(screen => $screen, font => $font, sound => $sound);
my $story = ColorDemo::Story->new(screen => $screen, sound => $sound);


my $retval = 0;

$start = time;
$retval = $intro->run();
if($retval == -1) {
    goto finish;
}
$scroller->run();
if($retval == -1) {
    goto finish;
}
$story->run();
if($retval == -1) {
    goto finish;
}

finish:
my $end = time;
$screen->reset();
$sound->finish();

print "\nRender took ", $end - $start, " seconds\n";

print "Done\n";
