package ColorDemo::Font;
use v5.38;
use strict;

use Term::ANSIScreen qw/:cursor :screen/;
use GD;
use Carp;

use Term::ReadKey qw(GetTerminalSize);
use Time::HiRes qw(time);

sub new($proto, %config) {
    my $class = ref($proto) || $proto;

    my $self = bless \%config, $class;

    if(!defined($self->{screen})) {
        croak("Font.pm needs screen");
    }

    my $screen = $self->{screen};
    ($self->{screenw}, $self->{screenh}) = $screen->getSize();

    ($self->{font}, $self->{fontw}, $self->{fonth}) = $screen->loadImage('gfx/c64.png');

    my @charmap = (
        '@abcdefghijklmnopqrstuvwxyz[§]^',
        ' !"#$%&\'()*+,-./0123456789:;<=>?',
        ' ABCDEFGHIJKLMNOPQRSTUVWXYZ',
    );

    my %chars;
    my $yoffs = 0;
    my ($r, $g, $b);
    foreach my $mapline (@charmap) {
        my $xoffs = 0;
        my @parts = split//, $mapline;
        foreach my $part (@parts) {
            my @pixels;
            for(my $y = 0; $y < 8; $y++) {
                for(my $x = 0; $x < 8; $x++) {
                    my $color = $self->{font}->getPixel($x + $xoffs, $y + $yoffs);
                    ($r,$g,$b) = $self->{font}->rgb($color);
                    if($r < 128) {
                        push @pixels, 1;
                    } else {
                        push @pixels, 0;
                    }
                }
            }
            if(!defined($chars{$part})) {
                $chars{$part} = \@pixels;
            }
            $xoffs += 8;
        }
        $yoffs += 8;
    }

    $self->{chars} = \%chars;

    return $self;
}

sub printChar($self, $x, $y, $val, $r = 255, $g = 255, $b = 255) {
    if(!defined($self->{chars}->{$val}) || $val eq ' ') {
        return;
    }

    for(my $xoffs = 0; $xoffs < 8; $xoffs++) {
        for(my $yoffs = 0; $yoffs < 8; $yoffs++) {
            if($self->{chars}->{$val}->[$xoffs + ($yoffs * 8)]) {
                $self->{screen}->setPixel($x + $xoffs, $y + $yoffs, $r, $g, $b);
            }
        }
    }

    return;
}

sub printMetalChar($self, $x, $y, $val, $r1 = 255, $g1 = 255, $b1 = 255, $r2 = 255, $g2 = 255, $b2 = 255) {
    if(!defined($self->{chars}->{$val}) || $val eq ' ') {
        return;
    }

    for(my $xoffs = 0; $xoffs < 8; $xoffs++) {
        for(my $yoffs = 0; $yoffs < 8; $yoffs++) {
            if($self->{chars}->{$val}->[$xoffs + ($yoffs * 8)]) {
                if(($xoffs + $yoffs) < 8) {
                    $self->{screen}->setPixel($x + $xoffs, $y + $yoffs, $r1, $g1, $b1);
                } else {
                    $self->{screen}->setPixel($x + $xoffs, $y + $yoffs, $r2, $g2, $b2);
                }
            }
        }
    }

    return;
}

sub printString($self, $x, $y, $val, $r = 255, $g = 255, $b = 255) {
    my $xoffs = 0;
    my @parts = split//, $val;

    foreach my $part (@parts) {
        $self->printChar($x + $xoffs, $y, $part, $r, $g, $b);
        #$self->printMetalChar($x + $xoffs, $y, $part, 0, 255, 255, 0, 255, 0);
        $xoffs += 8;
    }

    return;
}

sub renderCenterText($self, $val, $r, $g, $b) {
    my @lines = split/\n/, $val;

    my $height = 12 * scalar @lines;
    my $y = int(($self->{screenh} - $height) / 2);

    foreach my $line (@lines) {
        my $width = length($line) * 8;
        my $x = int(($self->{screenw} - $width) / 2);

        $self->printString($x, $y, $line, $r, $g, $b);
        $y += 12;
    }

    return;
}

