package ColorDemo::Scroller;
use v5.38;
use strict;

use Carp;

use Time::HiRes qw(time sleep);

sub new($proto, %config) {
    my $class = ref($proto) || $proto;

    my $self = bless \%config, $class;

    foreach my $name (qw[screen font sound]) {
        if(!defined($self->{$name})) {
            croak("Scroller.pm requires $name");
        }
    }

    my $screen = $self->{screen};
    ($self->{screenw}, $self->{screenh}) = $screen->getSize();

    ($self->{scrollerbg}, $self->{scrollerbgw}, $self->{scrollerbgh}) = $screen->loadImage('gfx/scrollerbg.png');

    $self->{music} = $self->{sound}->loadmusic('snd/scroller.ogg');


    return $self;
}

sub run($self) {
    $self->{screen}->setStatus("Obligatory scroller");
    $self->{sound}->playmusic($self->{music}, 0);

    my $text = 
        "                                       " . 
        "Let's do the obligatory scroller first. I'm not really sure why people do this, but nearly every demo seems to have one. " .
        "Oh well, at least i get to explain (roughly) how this demo program works. Basically, i have an array with one element per " . 
        "pixel. Each array element holds the red, green and blue value of that pixel. Simple enough. After filling out all the elements " . 
        "with the desired colors, i reset the cursor to the topleft position using some escape codes provided by Term::ANSIScreen. " . 
        "So far, so easy. To output the 'graphics' to the terminal, for each TWO rows of pixel data, i generate the per-column color " . 
        "escape codes (foreground for one row, background for the other) and then print a Unicode HALFBLOCK symbol. This way, i can " .
        "use double the vertical resolution of the terminal and give each pixel a somewhat more square appearance.            " .
        "Ok, i think i have enough text, time to end this scroller!" .
        "                                       ";


    my $xoffs = 0;
    my @chars = split//, $text;

    my $lasttime = time;

    my $fullprogress = 270 + (length($text) * 8);

    # White flash at the start
    while(1) {
        #$self->{screen}->clearBuffer(0, 0, 0);
        $self->{screen}->setProgress($xoffs / $fullprogress);
        my $fade = (sin($xoffs / 70) * 0.4) + 0.6;
        $self->{screen}->setImage(0, 0, $self->{scrollerbg}, $fade);

        my $chroffs = int($xoffs / 8);
        if($chroffs > scalar @chars) {
            last;
        }
        for(my $idx = 0; $idx < scalar @chars; $idx++) {
            my $x = ($idx * 8) - (int($xoffs % 8) || 0);
            my $realchroffs = $chroffs + $idx;

            my $y = int(($self->{screenh} / 2) + (sin(($xoffs - $realchroffs) / 30) * 10) + (cos($xoffs / 60) * 40)) - 4;

            if($x < -8 || $x > $self->{screenw} || $realchroffs < 0 || $realchroffs >= scalar @chars) {
                next;
            }

            $self->{font}->printMetalChar($x, $y, $chars[$realchroffs], 255, 255, 0, 0, 255, 255);
        }

        my $key = $self->{screen}->render();
        if($key) {
            $self->{screen}->setProgress(-1);
            $self->{sound}->stopmusic();
            return $key;
        }
        my $now = time;
        my $duration = $now - $lasttime;
        $lasttime = $now;
        my $stepsize = 91 / (1 / $duration);
        $xoffs += $stepsize;
    }


    $self->{screen}->setProgress(-1);
    $self->{sound}->stopmusic();
    return 0;
}
