package ColorDemo::Sound;
use v5.38;
use strict;

use Carp;

use Time::HiRes qw(time);

use SDL;
use SDL::Mixer;
use SDL::Mixer::Channels;
use SDL::Mixer::Samples;
use SDL::Mixer::Music;

sub new($proto, %config) {
    my $class = ref($proto) || $proto;

    my $self = bless \%config, $class;

    if(0) {
        foreach my $name (qw[screen font]) {
            if(!defined($self->{$name})) {
                croak("Intro.pm requires $name");
            }
        }
    }

    # make more than the default 2 channels, we make 8, but need 3
    #printf("We got %d channels!\n", SDL::Mixer::Channels::allocate_channels( 8 ) );

    SDL::init(SDL_INIT_AUDIO);
                                                       #mono 1 or stereo 2
    SDL::Mixer::open_audio(44100, SDL::Constants::AUDIO_S16, 2, 4096 );

    foreach my $chan(1..3){
       SDL::Mixer::Channels::volume($chan, 128 ); # set max volume
     }

    return $self;
}

sub finish($self) {
    SDL::Mixer::close_audio();
}

sub loadfx($self, $fname) {
    my $fx = SDL::Mixer::Samples::load_WAV($fname);

    return $fx;
}


sub playfx($self, $channel, $fx, $loop) {
    my $mode = 0;
    if($loop) {
        $mode = -1;
    }

    SDL::Mixer::Channels::play_channel($channel, $fx, $mode );
    return;
}

sub stopfx($self, $channel) {
    SDL::Mixer::Channels::halt_channel($channel);
    return;
}

sub loadmusic($self, $fname) {
    my $mus = SDL::Mixer::Music::load_MUS($fname);

    return $mus;
}

sub playmusic($self, $music, $loop) {
    my $mode = 0;
    if($loop) {
        $mode = -1;
    }
    SDL::Mixer::Music::play_music( $music, $mode );
    return;
}

sub stopmusic($self) {
    SDL::Mixer::Music::halt_music();
    return;
}
