package ColorDemo::Story;
use v5.38;
use strict;

use Carp;

use Time::HiRes qw(time sleep);
use Data::Dumper;

sub new($proto, %config) {
    my $class = ref($proto) || $proto;

    my $self = bless \%config, $class;

    foreach my $name (qw[screen sound]) {
        if(!defined($self->{$name})) {
            croak("Story.pm requires $name");
        }
    }

    my $screen = $self->{screen};
    ($self->{screenw}, $self->{screenh}) = $screen->getSize();

    my @storyboard = (
        ['01', 2.5],
        ['02', 3.5],
        ['03', 4.0],
        ['04', 5.0],
        ['05', 8.0],
        ['06', 10.0],
        ['07', 10.0],
        ['08', 4.0],
        ['09', 5.0],
        ['10', 10.0],
        ['11', 10.0],
        ['12', 2.0],
        ['13', 4.5],
        ['14', 13.5],
        ['15', 1.5],
    );

    for(my $i = 0; $i < scalar @storyboard; $i++) {
        my $pngname = 'gfx/story_' . $storyboard[$i]->[0] . '.png';
        my $fxname = 'snd/story_' . $storyboard[$i]->[0] . '.wav';


        my ($img, $w, $h) = $screen->loadImage($pngname);
        push @{$storyboard[$i]}, $img;

        if(-f $fxname) {
            my $fx = $self->{sound}->loadfx($fxname);
            push @{$storyboard[$i]}, $fx;
        }

        #print "\n\n", Dumper($storyboard[$i]);
        #print "\n$pngname\n";
        #croak("BLA");
    }
    $self->{storyboard} = \@storyboard;

    $self->{music} = $self->{sound}->loadmusic('snd/heart_of_nowhere.ogg');

    return $self;
}

sub run($self) {
    my $screen = $self->{screen};
    my $sound = $self->{sound};

    $self->{screen}->setStatus("Lord of the Languages");
    $self->{sound}->playmusic($self->{music}, 0);

    my $lastimage;
    foreach my $part (@{$self->{storyboard}}) {
        my ($name, $totalduration, $img, $fx) = @{$part};

        if(defined($fx)) {
            $self->{sound}->playfx(2, $fx, 0);
        }

        if($name eq '02') {
            $screen->setInfo('MUSIC: "Heart of Nowhere" Kevin MacLeod (incompetech.com)');
        } elsif($name eq '05') {
            $screen->setInfo('MUSIC: Licensed under Creative Commons: By Attribution 4.0 License');
        } elsif($name eq '08') {
            $screen->setInfo('MUSIC: http://creativecommons.org/licenses/by/4.0/');
        } elsif($name eq '10') {
            $screen->setInfo('');
        }

        my $fade = 0.0;
        my $lasttime = time;
        my $endtime = time + $totalduration;
        while(1) {
            if(!defined($lastimage)) {
                $screen->setImage(0, 0, $img, $fade);
            } else {
                $screen->crossFadeImage(0, 0, $lastimage, $img, $fade);
            }
            my $key = $screen->render();
            if($key) {
                $self->{sound}->stopmusic();
                return $key;
            }
            my $now = time;
            my $duration = $now - $lasttime;
            $lasttime = $now;
            if($fade >= 1.0) {
                last;
            }
            my $stepsize = 1.0 / (1.0 / $duration);
            $fade+= $stepsize;
        }
        $lastimage = $img;

        while(1) {
            my $key = $screen->render();
            if($key) {
                $self->{sound}->stopmusic();
                return $key;
            }
            if(time > $endtime) {
                last;
            }
        }
    }

    $self->{sound}->stopmusic();
    return 0;
}
